@@ -67,7 +67,7 @@ def majority_vote_with_distortion_score_weighting(
                 cc_data.shape[0],
             ]
         )
-    if len(set([x[1] for x in consensus_clusters])) == 1:
+    if len(set([x[1] for x in consensus_cluster_score])) == 1:
         # All consensus clusters have equal scores, return consensus cluster label with the greatest number of votes
         return sorted(consensus_cluster_score, key=lambda x: x[2])[::-1][0][0]
     # Return consensus cluster label with the highest score
@@ -88,7 +88,7 @@ def majority_vote_with_distance_weighting(
     for cid, cc_data in consensus_clusters.loc[clusters].groupby("cluster_label"):
         scores = sum([getattr(distance, metric)(row[features], cc_row[features]) for _, cc_row in cc_data.iterrows()])
         consensus_cluster_score.append([cid, cc_data.shape[0] / (scores / cc_data.shape[0]), cc_data.shape[0]])
-    if len(set([x[1] for x in consensus_clusters])) == 1:
+    if len(set([x[1] for x in consensus_cluster_score])) == 1:
         # All consensus clusters have equal scores, return consensus cluster label with the greatest number of votes
         return sorted(consensus_cluster_score, key=lambda x: x[2])[::-1][0][0]
     # Return consensus cluster label with the highest score
@@ -119,12 +119,13 @@ class EnsembleClustering(Clustering):
             self,
             data: pd.DataFrame,
             features: List[str],
-            labels: Dict[str, Union[pd.Series, np.ndarray]],
-            ignore_clusters: Optional[List[str]] = None,
-            *args,
-            **kwargs
+            verbose: bool = True,
+            random_state: int = 42,
+            n_sources: Optional[Dict] = None,
+            pre_embedded: bool = False,
+            labels: Optional[Dict[str, Union[pd.Series, np.ndarray]]] = None
     ):
-        super().__init__(data, features, *args, **kwargs)
+        super().__init__(data, features, verbose, random_state, n_sources, pre_embedded, labels)
         self.original_cluster_labels = labels
         cluster_membership = pd.concat(
             [
@@ -133,8 +134,6 @@ class EnsembleClustering(Clustering):
             ],
             axis=1
         )
-        if ignore_clusters:
-            cluster_membership.drop(ignore_clusters, axis=1, inplace=True)
         self.data = pd.concat([self.data, cluster_membership], axis=1)
         self.clusters = cluster_membership.columns.tolist()
         self._cluster_weights = {}
